import os
import tensorflow as tf
from keras.models import load_model
from .Config import Config

class Brain:

    def __init__(self, mode=0):
        self.modeSwitcher = {
            0: Config.get("CNN_DEFAULT_MODEL")
        }
        self.model = None
        self.init(self.modeSwitcher[mode])
    def init(self, mode):
        tf.get_logger().setLevel('ERROR')
        tf.autograph.set_verbosity(Config.get("NORMALIZATION_ENABLED"))
        self.model = load_model(mode)

    def predict(self, image):
        prediction = self.model.predict(image)
        result = {
            True: Config.get("RESULT_MAP")["WANDER"],
            False: Config.get("RESULT_MAP")["NORMAL"]
        }
        prediction = [prediction > Config.get("RESULT_THRESHOLD")]
        return result[prediction[0][0][0]]

    def predict_profiler(self, image):
        tf.profiler.experimental.start(Config("LOGS_FOLDER"))
        self.model.predict(image)
        tf.profiler.experimental.stop()


import base64
import numpy as np
from PIL import Image as PilImage
from io import BytesIO
from .Config import Config


class Image:
    def __init__(self, image=None, path=""):
        if image: 
            self.image = image
        if path:
            self.image = self.seek(path)

        self.process()

    def get(self):
        return self.image

    
    def decode(self, image):
        image_bytes = base64.b64decode(image)
        image = BytesIO(image_bytes)
        return PilImage.open(image).convert('L')

    def encode(self, image):
        image = base64.b64encode(image)
        return image
    

    def resize(self,image):
        image = np.uint8(image)
        image = PilImage.fromarray(image).resize((Config.get("IMAGE_WIDTH"), Config.get("IMAGE_HEIGHT")))
        return np.array(image)

    def normalize(self, image):
        return image / 255

    def process(self):
        image = self.decode(self.image)
        image = np.array(image)
        if Config.get("RESIZE_ENABLED"):
            image = self.resize(image)
        if Config.get("NORMALIZATION_ENABLED"):
            image = self.normalize(image)
        image = image[np.newaxis, ..., np.newaxis]
        self.image = image
        return image

    def seek(self, path):
        image = PilImage.open(path).convert('L')
        im_file = BytesIO()
        image.save(im_file, format="JPEG")
        return self.encode(im_file.getvalue())

    


from pkg_resources import resource_filename, Requirement
class ConfigEnum():
    dict = {
        "IMAGE_HEIGHT": 128,
        "IMAGE_WIDTH": 128,
        "LOGS_FOLDER": "logs",
        "RESULT_THRESHOLD": 0.5,
        "RESULT_MAP": {
            "WANDER": 1,
            "NORMAL": 0
        },
        "VERBOSITY": 1,
        "RESIZE_ENABLED": True,
        "NORMALIZATION_ENABLED": True,
        "CNN_DEFAULT_MODEL": resource_filename(__name__, 'model/modelCNN-1605022224.h5')
    }

    def __new__(key):
        return ConfigEnum.get(key)

    def get(key):
        return ConfigEnum.dict[key]
    
    def set(key, value):
        if key in ConfigEnum.dict.keys():
            ConfigEnum.dict[key] = value

Config = ConfigEnum



from setuptools import setup, find_packages
import codecs
import os

VERSION = '0.0.32'
DESCRIPTION = 'Elderly wandering prediction AI'
LONG_DESCRIPTION = 'Originally from the paper ''Convolutional Neural Network for Elderly Wandering Prediction in indoor scenarios.'' An AI to detect elderly wandering by movementation in indoor scenarios.'

# Setting up
setup(
    name="smartcare",
    version=VERSION,
    author="Rafael Faustini",
    author_email="<contato@rafaelfaustini.com.br>",
    description=DESCRIPTION,
    package_data={'smartcare': ['model/*']},
    include_package_data=True,
    long_description_content_type="text/markdown",
    long_description=LONG_DESCRIPTION,
    packages=find_packages(),
    install_requires=['tensorflow', 'keras', 'pillow', 'tensorboard_plugin_profile'],
    keywords=['machinelearning', 'paper', 'smartcare', 'cnn', 'artificial inteligence', 'alzheimer', 'health'],
    classifiers=[
        "Development Status :: 1 - Planning",
        "Intended Audience :: Developers",
        "Programming Language :: Python :: 3",
        "Operating System :: Unix",
        "Operating System :: MacOS :: MacOS X",
        "Operating System :: Microsoft :: Windows",
    ]
)